@@ -38,7 +38,7 @@ impl<'src> Analyzer<'src> {
     let mut define = |name: Name<'src>,
                       second_type: &'static str,
                       duplicates_allowed: bool|
-     -> CompileResult<'src, ()> {
+     -> CompileResult<'src> {
       if let Some((first_type, original)) = definitions.get(name.lexeme()) {
         if !(*first_type == second_type && duplicates_allowed) {
           let (original, redefinition) = if name.line < original.line {
@@ -47,7 +47,7 @@ impl<'src> Analyzer<'src> {
             (*original, name)
           };
 
-          return Err(redefinition.token().error(Redefinition {
+          return Err(redefinition.token.error(Redefinition {
             first_type,
             second_type,
             name: name.lexeme(),
@@ -75,17 +75,18 @@ impl<'src> Analyzer<'src> {
           }
           Item::Comment(_) => (),
           Item::Import { absolute, .. } => {
-            stack.push(asts.get(absolute.as_ref().unwrap()).unwrap());
+            if let Some(absolute) = absolute {
+              stack.push(asts.get(absolute).unwrap());
+            }
           }
-          Item::Mod { absolute, name, .. } => {
-            define(*name, "module", false)?;
-            modules.insert(
-              name.to_string(),
-              (
-                *name,
-                Self::analyze(loaded, paths, asts, absolute.as_ref().unwrap())?,
-              ),
-            );
+          Item::Module { absolute, name, .. } => {
+            if let Some(absolute) = absolute {
+              define(*name, "module", false)?;
+              modules.insert(
+                name.lexeme().into(),
+                (*name, Self::analyze(loaded, paths, asts, absolute)?),
+              );
+            }
           }
           Item::Recipe(recipe) => {
             if recipe.enabled() {
@@ -153,13 +154,13 @@ impl<'src> Analyzer<'src> {
     })
   }
 
-  fn analyze_recipe(recipe: &UnresolvedRecipe<'src>) -> CompileResult<'src, ()> {
+  fn analyze_recipe(recipe: &UnresolvedRecipe<'src>) -> CompileResult<'src> {
     let mut parameters = BTreeSet::new();
     let mut passed_default = false;
 
     for parameter in &recipe.parameters {
       if parameters.contains(parameter.name.lexeme()) {
-        return Err(parameter.name.token().error(DuplicateParameter {
+        return Err(parameter.name.token.error(DuplicateParameter {
           recipe: recipe.name.lexeme(),
           parameter: parameter.name.lexeme(),
         }));
@@ -172,7 +173,7 @@ impl<'src> Analyzer<'src> {
         return Err(
           parameter
             .name
-            .token()
+            .token
             .error(RequiredParameterFollowsDefaultParameter {
               parameter: parameter.name.lexeme(),
             }),
@@ -198,23 +199,23 @@ impl<'src> Analyzer<'src> {
     Ok(())
   }
 
-  fn analyze_assignment(&self, assignment: &Assignment<'src>) -> CompileResult<'src, ()> {
+  fn analyze_assignment(&self, assignment: &Assignment<'src>) -> CompileResult<'src> {
     if self.assignments.contains_key(assignment.name.lexeme()) {
-      return Err(assignment.name.token().error(DuplicateVariable {
+      return Err(assignment.name.token.error(DuplicateVariable {
         variable: assignment.name.lexeme(),
       }));
     }
     Ok(())
   }
 
-  fn analyze_alias(alias: &Alias<'src, Name<'src>>) -> CompileResult<'src, ()> {
+  fn analyze_alias(alias: &Alias<'src, Name<'src>>) -> CompileResult<'src> {
     let name = alias.name.lexeme();
 
-    for attr in &alias.attributes {
-      if *attr != Attribute::Private {
-        return Err(alias.name.token().error(AliasInvalidAttribute {
+    for attribute in &alias.attributes {
+      if *attribute != Attribute::Private {
+        return Err(alias.name.token.error(AliasInvalidAttribute {
           alias: name,
-          attr: *attr,
+          attribute: attribute.clone(),
         }));
       }
     }
@@ -222,7 +223,7 @@ impl<'src> Analyzer<'src> {
     Ok(())
   }
 
-  fn analyze_set(&self, set: &Set<'src>) -> CompileResult<'src, ()> {
+  fn analyze_set(&self, set: &Set<'src>) -> CompileResult<'src> {
     if let Some(original) = self.sets.get(set.name.lexeme()) {
       return Err(set.name.error(DuplicateSet {
         setting: original.name.lexeme(),
@@ -237,10 +238,9 @@ impl<'src> Analyzer<'src> {
     recipes: &Table<'src, Rc<Recipe<'src>>>,
     alias: Alias<'src, Name<'src>>,
   ) -> CompileResult<'src, Alias<'src>> {
-    let token = alias.name.token();
     // Make sure the alias doesn't conflict with any recipe
     if let Some(recipe) = recipes.get(alias.name.lexeme()) {
-      return Err(token.error(AliasShadowsRecipe {
+      return Err(alias.name.token.error(AliasShadowsRecipe {
         alias: alias.name.lexeme(),
         recipe_line: recipe.line_number(),
       }));
@@ -249,7 +249,7 @@ impl<'src> Analyzer<'src> {
     // Make sure the target recipe exists
     match recipes.get(alias.target.lexeme()) {
       Some(target) => Ok(alias.resolve(Rc::clone(target))),
-      None => Err(token.error(UnknownAliasTarget {
+      None => Err(alias.name.token.error(UnknownAliasTarget {
         alias: alias.name.lexeme(),
         target: alias.target.lexeme(),
       })),
