@@ -23,35 +23,35 @@ use {super::*, TokenKind::*};
 /// find it, it adds that token to the set. When the parser accepts a token, the
 /// set is cleared. If the parser finds a token which is unexpected, the
 /// contents of the set is printed in the resultant error message.
-pub(crate) struct Parser<'tokens, 'src> {
-  /// Source tokens
-  tokens: &'tokens [Token<'src>],
-  /// Index of the next un-parsed token
-  next: usize,
-  /// Current expected tokens
-  expected: BTreeSet<TokenKind>,
-  /// Current recursion depth
-  depth: usize,
-  /// Path to the file being parsed
-  path: PathBuf,
-  /// Depth of submodule being parsed
-  submodule: u32,
+pub(crate) struct Parser<'run, 'src> {
+  expected_tokens: BTreeSet<TokenKind>,
+  file_path: &'run Path,
+  module_namepath: &'run Namepath<'src>,
+  next_token: usize,
+  recursion_depth: usize,
+  submodule_depth: u32,
+  tokens: &'run [Token<'src>],
+  working_directory: &'run Path,
 }
 
-impl<'tokens, 'src> Parser<'tokens, 'src> {
+impl<'run, 'src> Parser<'run, 'src> {
   /// Parse `tokens` into an `Ast`
   pub(crate) fn parse(
-    submodule: u32,
-    path: &Path,
-    tokens: &'tokens [Token<'src>],
+    file_path: &'run Path,
+    module_namepath: &'run Namepath<'src>,
+    submodule_depth: u32,
+    tokens: &'run [Token<'src>],
+    working_directory: &'run Path,
   ) -> CompileResult<'src, Ast<'src>> {
-    Parser {
-      depth: 0,
-      expected: BTreeSet::new(),
-      next: 0,
-      path: path.into(),
-      submodule,
+    Self {
+      expected_tokens: BTreeSet::new(),
+      file_path,
+      module_namepath,
+      next_token: 0,
+      recursion_depth: 0,
+      submodule_depth,
       tokens,
+      working_directory,
     }
     .parse_ast()
   }
@@ -65,7 +65,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   fn unexpected_token(&self) -> CompileResult<'src, CompileError<'src>> {
     self.error(CompileErrorKind::UnexpectedToken {
       expected: self
-        .expected
+        .expected_tokens
         .iter()
         .copied()
         .filter(|kind| *kind != ByteOrderMark)
@@ -81,8 +81,8 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// An iterator over the remaining significant tokens
-  fn rest(&self) -> impl Iterator<Item = Token<'src>> + 'tokens {
-    self.tokens[self.next..]
+  fn rest(&self) -> impl Iterator<Item = Token<'src>> + 'run {
+    self.tokens[self.next_token..]
       .iter()
       .copied()
       .filter(|token| token.kind != Whitespace)
@@ -107,7 +107,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   /// The first token in `kinds` will be added to the expected token set.
   fn next_are(&mut self, kinds: &[TokenKind]) -> bool {
     if let Some(&kind) = kinds.first() {
-      self.expected.insert(kind);
+      self.expected_tokens.insert(kind);
     }
 
     let mut rest = self.rest();
@@ -126,10 +126,10 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
 
   /// Advance past one significant token, clearing the expected token set.
   fn advance(&mut self) -> CompileResult<'src, Token<'src>> {
-    self.expected.clear();
+    self.expected_tokens.clear();
 
-    for skipped in &self.tokens[self.next..] {
-      self.next += 1;
+    for skipped in &self.tokens[self.next_token..] {
+      self.next_token += 1;
 
       if skipped.kind != Whitespace {
         return Ok(*skipped);
@@ -150,7 +150,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Return an unexpected token error if the next token is not an EOL
-  fn expect_eol(&mut self) -> CompileResult<'src, ()> {
+  fn expect_eol(&mut self) -> CompileResult<'src> {
     self.accept(Comment)?;
 
     if self.next_is(Eof) {
@@ -160,7 +160,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     self.expect(Eol).map(|_| ())
   }
 
-  fn expect_keyword(&mut self, expected: Keyword) -> CompileResult<'src, ()> {
+  fn expect_keyword(&mut self, expected: Keyword) -> CompileResult<'src> {
     let found = self.advance()?;
 
     if found.kind == Identifier && expected == found.lexeme() {
@@ -175,7 +175,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
 
   /// Return an internal error if the next token is not of kind `Identifier`
   /// with lexeme `lexeme`.
-  fn presume_keyword(&mut self, keyword: Keyword) -> CompileResult<'src, ()> {
+  fn presume_keyword(&mut self, keyword: Keyword) -> CompileResult<'src> {
     let next = self.advance()?;
 
     if next.kind != Identifier {
@@ -231,7 +231,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Return an error if the next token is of kind `forbidden`
-  fn forbid<F>(&self, forbidden: TokenKind, error: F) -> CompileResult<'src, ()>
+  fn forbid<F>(&self, forbidden: TokenKind, error: F) -> CompileResult<'src>
   where
     F: FnOnce(Token) -> CompileError,
   {
@@ -334,31 +334,43 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
             self.presume_keyword(Keyword::Export)?;
             items.push(Item::Assignment(self.parse_assignment(true)?));
           }
-          Some(Keyword::Import) if self.next_are(&[Identifier, StringToken]) => {
+          Some(Keyword::Import)
+            if self.next_are(&[Identifier, StringToken])
+              || self.next_are(&[Identifier, QuestionMark]) =>
+          {
             self.presume_keyword(Keyword::Import)?;
+            let optional = self.accepted(QuestionMark)?;
+            let (path, relative) = self.parse_string_literal_token()?;
             items.push(Item::Import {
-              relative: self.parse_string_literal()?,
               absolute: None,
+              optional,
+              path,
+              relative,
             });
           }
           Some(Keyword::Mod)
             if self.next_are(&[Identifier, Identifier, StringToken])
               || self.next_are(&[Identifier, Identifier, Eof])
-              || self.next_are(&[Identifier, Identifier, Eol]) =>
+              || self.next_are(&[Identifier, Identifier, Eol])
+              || self.next_are(&[Identifier, QuestionMark]) =>
           {
             self.presume_keyword(Keyword::Mod)?;
+
+            let optional = self.accepted(QuestionMark)?;
+
             let name = self.parse_name()?;
 
-            let path = if self.next_is(StringToken) {
+            let relative = if self.next_is(StringToken) {
               Some(self.parse_string_literal()?)
             } else {
               None
             };
 
-            items.push(Item::Mod {
-              name,
+            items.push(Item::Module {
               absolute: None,
-              path,
+              name,
+              optional,
+              relative,
             });
           }
           Some(Keyword::Set)
@@ -407,7 +419,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       }
     }
 
-    if self.next == self.tokens.len() {
+    if self.next_token == self.tokens.len() {
       Ok(Ast {
         warnings: Vec::new(),
         items,
@@ -415,7 +427,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     } else {
       Err(self.internal_error(format!(
         "Parse completed with {} unparsed tokens",
-        self.tokens.len() - self.next,
+        self.tokens.len() - self.next_token,
       ))?)
     }
   }
@@ -423,7 +435,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   /// Parse an alias, e.g `alias name := target`
   fn parse_alias(
     &mut self,
-    attributes: BTreeSet<Attribute>,
+    attributes: BTreeSet<Attribute<'src>>,
   ) -> CompileResult<'src, Alias<'src, Name<'src>>> {
     self.presume_keyword(Keyword::Alias)?;
     let name = self.parse_name()?;
@@ -452,7 +464,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
 
   /// Parse an expression, e.g. `1 + 2`
   fn parse_expression(&mut self) -> CompileResult<'src, Expression<'src>> {
-    if self.depth == if cfg!(windows) { 48 } else { 256 } {
+    if self.recursion_depth == if cfg!(windows) { 48 } else { 256 } {
       let token = self.next()?;
       return Err(CompileError::new(
         token,
@@ -460,7 +472,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       ));
     }
 
-    self.depth += 1;
+    self.recursion_depth += 1;
 
     let expression = if self.accepted_keyword(Keyword::If)? {
       self.parse_conditional()?
@@ -484,7 +496,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       }
     };
 
-    self.depth -= 1;
+    self.recursion_depth -= 1;
 
     Ok(expression)
   }
@@ -574,8 +586,10 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     }
   }
 
-  /// Parse a string literal, e.g. `"FOO"`
-  fn parse_string_literal(&mut self) -> CompileResult<'src, StringLiteral<'src>> {
+  /// Parse a string literal, e.g. `"FOO"`, returning the string literal and the string token
+  fn parse_string_literal_token(
+    &mut self,
+  ) -> CompileResult<'src, (Token<'src>, StringLiteral<'src>)> {
     let token = self.expect(StringToken)?;
 
     let kind = StringKind::from_string_or_backtick(token)?;
@@ -620,7 +634,13 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       unindented
     };
 
-    Ok(StringLiteral { kind, raw, cooked })
+    Ok((token, StringLiteral { kind, raw, cooked }))
+  }
+
+  /// Parse a string literal, e.g. `"FOO"`
+  fn parse_string_literal(&mut self) -> CompileResult<'src, StringLiteral<'src>> {
+    let (_token, string_literal) = self.parse_string_literal_token()?;
+    Ok(string_literal)
   }
 
   /// Parse a name from an identifier token
@@ -652,7 +672,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
     &mut self,
     doc: Option<&'src str>,
     quiet: bool,
-    attributes: BTreeSet<Attribute>,
+    attributes: BTreeSet<Attribute<'src>>,
   ) -> CompileResult<'src, UnresolvedRecipe<'src>> {
     let name = self.parse_name()?;
 
@@ -717,14 +737,16 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       attributes,
       body,
       dependencies,
+      depth: self.submodule_depth,
       doc,
+      file_path: self.file_path.into(),
       name,
+      namepath: self.module_namepath.join(name),
       parameters: positional.into_iter().chain(variadic).collect(),
-      path: self.path.clone(),
       priors,
       private: name.lexeme().starts_with('_'),
       quiet,
-      depth: self.submodule,
+      working_directory: self.working_directory.into(),
     })
   }
 
@@ -830,6 +852,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
       Keyword::Fallback => Some(Setting::Fallback(self.parse_set_bool()?)),
       Keyword::IgnoreComments => Some(Setting::IgnoreComments(self.parse_set_bool()?)),
       Keyword::PositionalArguments => Some(Setting::PositionalArguments(self.parse_set_bool()?)),
+      Keyword::Quiet => Some(Setting::Quiet(self.parse_set_bool()?)),
       Keyword::WindowsPowershell => Some(Setting::WindowsPowerShell(self.parse_set_bool()?)),
       _ => None,
     };
@@ -882,7 +905,7 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
   }
 
   /// Parse recipe attributes
-  fn parse_attributes(&mut self) -> CompileResult<'src, Option<BTreeSet<Attribute>>> {
+  fn parse_attributes(&mut self) -> CompileResult<'src, Option<BTreeSet<Attribute<'src>>>> {
     let mut attributes = BTreeMap::new();
 
     while self.accepted(BracketL)? {
@@ -899,6 +922,15 @@ impl<'tokens, 'src> Parser<'tokens, 'src> {
             first: *line,
           }));
         }
+
+        let attribute = if self.accepted(ParenL)? {
+          let argument = self.parse_string_literal()?;
+          self.expect(ParenR)?;
+          attribute.with_argument(name, argument)?
+        } else {
+          attribute
+        };
+
         attributes.insert(attribute, name.line);
 
         if !self.accepted(Comma)? {
@@ -942,7 +974,14 @@ mod tests {
   fn test(text: &str, want: Tree) {
     let unindented = unindent(text);
     let tokens = Lexer::test_lex(&unindented).expect("lexing failed");
-    let justfile = Parser::parse(0, &PathBuf::new(), &tokens).expect("parsing failed");
+    let justfile = Parser::parse(
+      &PathBuf::new(),
+      &Namepath::default(),
+      0,
+      &tokens,
+      &PathBuf::new(),
+    )
+    .expect("parsing failed");
     let have = justfile.tree();
     if have != want {
       println!("parsed text: {unindented}");
@@ -980,7 +1019,13 @@ mod tests {
   ) {
     let tokens = Lexer::test_lex(src).expect("Lexing failed in parse test...");
 
-    match Parser::parse(0, &PathBuf::new(), &tokens) {
+    match Parser::parse(
+      &PathBuf::new(),
+      &Namepath::default(),
+      0,
+      &tokens,
+      &PathBuf::new(),
+    ) {
       Ok(_) => panic!("Parsing unexpectedly succeeded"),
       Err(have) => {
         let want = CompileError {
@@ -1892,6 +1937,24 @@ mod tests {
     tree: (justfile (set positional_arguments true)),
   }
 
+  test! {
+    name: set_quiet_implicit,
+    text: "set quiet",
+    tree: (justfile (set quiet true)),
+  }
+
+  test! {
+    name: set_quiet_true,
+    text: "set quiet := true",
+    tree: (justfile (set quiet true)),
+  }
+
+  test! {
+    name: set_quiet_false,
+    text: "set quiet := false",
+    tree: (justfile (set quiet false)),
+  }
+
   test! {
     name: set_positional_arguments_false,
     text: "set positional-arguments := false",
@@ -2000,6 +2063,36 @@ mod tests {
     tree: (justfile (import "some/file/path.txt")),
   }
 
+  test! {
+    name: optional_import,
+    text: "import? \"some/file/path.txt\"     \n",
+    tree: (justfile (import ? "some/file/path.txt")),
+  }
+
+  test! {
+    name: module_with,
+    text: "mod foo",
+    tree: (justfile (mod foo )),
+  }
+
+  test! {
+    name: optional_module,
+    text: "mod? foo",
+    tree: (justfile (mod ? foo)),
+  }
+
+  test! {
+    name: module_with_path,
+    text: "mod foo \"some/file/path.txt\"     \n",
+    tree: (justfile (mod foo "some/file/path.txt")),
+  }
+
+  test! {
+    name: optional_module_with_path,
+    text: "mod? foo \"some/file/path.txt\"     \n",
+    tree: (justfile (mod ? foo "some/file/path.txt")),
+  }
+
   error! {
     name:   alias_syntax_multiple_rhs,
     input:  "alias foo := bar baz",
