@@ -40,6 +40,6 @@ impl Verbosity {
   }
 
   pub const fn default() -> Self {
-    Self::Taciturn
+    Taciturn
   }
 }
