@@ -52,6 +52,74 @@ fn module_recipes_can_be_run_as_subcommands() {
     .run();
 }
 
+#[test]
+fn module_recipes_can_be_run_with_path_syntax() {
+  Test::new()
+    .write("foo.just", "foo:\n @echo FOO")
+    .justfile(
+      "
+        mod foo
+      ",
+    )
+    .test_round_trip(false)
+    .arg("--unstable")
+    .arg("foo::foo")
+    .stdout("FOO\n")
+    .run();
+}
+
+#[test]
+fn nested_module_recipes_can_be_run_with_path_syntax() {
+  Test::new()
+    .write("foo.just", "mod bar")
+    .write("bar.just", "baz:\n @echo BAZ")
+    .justfile(
+      "
+        mod foo
+      ",
+    )
+    .test_round_trip(false)
+    .arg("--unstable")
+    .arg("foo::bar::baz")
+    .stdout("BAZ\n")
+    .run();
+}
+
+#[test]
+fn invalid_path_syntax() {
+  Test::new()
+    .test_round_trip(false)
+    .arg(":foo::foo")
+    .stderr("error: Justfile does not contain recipe `:foo::foo`.\n")
+    .status(EXIT_FAILURE)
+    .run();
+
+  Test::new()
+    .test_round_trip(false)
+    .arg("foo::foo:")
+    .stderr("error: Justfile does not contain recipe `foo::foo:`.\n")
+    .status(EXIT_FAILURE)
+    .run();
+
+  Test::new()
+    .test_round_trip(false)
+    .arg("foo:::foo")
+    .stderr("error: Justfile does not contain recipe `foo:::foo`.\n")
+    .status(EXIT_FAILURE)
+    .run();
+}
+
+#[test]
+fn missing_recipe_after_invalid_path() {
+  Test::new()
+    .test_round_trip(false)
+    .arg(":foo::foo")
+    .arg("bar")
+    .stderr("error: Justfile does not contain recipes `:foo::foo` or `bar`.\n")
+    .status(EXIT_FAILURE)
+    .run();
+}
+
 #[test]
 fn assignments_are_evaluated_in_modules() {
   Test::new()
@@ -163,10 +231,10 @@ fn modules_use_module_settings() {
     .stderr(
       "
       error: Recipe `foo` first defined on line 2 is redefined on line 3
-       --> foo.just:3:1
-        |
-      3 | foo:
-        | ^^^
+       ——▶ foo.just:3:1
+        │
+      3 │ foo:
+        │ ^^^
     ",
     )
     .run();
@@ -185,10 +253,10 @@ fn modules_conflict_with_recipes() {
     .stderr(
       "
       error: Module `foo` defined on line 1 is redefined as a recipe on line 2
-       --> justfile:2:1
-        |
-      2 | foo:
-        | ^^^
+       ——▶ justfile:2:1
+        │
+      2 │ foo:
+        │ ^^^
     ",
     )
     .test_round_trip(false)
@@ -211,10 +279,10 @@ fn modules_conflict_with_aliases() {
     .stderr(
       "
       error: Module `foo` defined on line 1 is redefined as an alias on line 3
-       --> justfile:3:7
-        |
-      3 | alias foo := bar
-        |       ^^^
+       ——▶ justfile:3:7
+        │
+      3 │ alias foo := bar
+        │       ^^^
     ",
     )
     .test_round_trip(false)
@@ -240,10 +308,10 @@ fn modules_conflict_with_other_modules() {
     .stderr(
       "
       error: Module `foo` first defined on line 1 is redefined on line 2
-       --> justfile:2:5
-        |
-      2 | mod foo
-        |     ^^^
+       ——▶ justfile:2:5
+        │
+      2 │ mod foo
+        │     ^^^
     ",
     )
     .arg("--unstable")
@@ -266,6 +334,22 @@ fn modules_are_dumped_correctly() {
     .run();
 }
 
+#[test]
+fn optional_modules_are_dumped_correctly() {
+  Test::new()
+    .write("foo.just", "foo:\n @echo FOO")
+    .justfile(
+      "
+        mod? foo
+      ",
+    )
+    .test_round_trip(false)
+    .arg("--unstable")
+    .arg("--dump")
+    .stdout("mod? foo\n")
+    .run();
+}
+
 #[test]
 fn modules_can_be_in_subdirectory() {
   Test::new()
@@ -350,10 +434,10 @@ fn modules_require_unambiguous_file() {
     .stderr(
       "
       error: Found multiple source files for module `foo`: `foo.just` and `foo/justfile`
-       --> justfile:1:5
-        |
-      1 | mod foo
-        |     ^^^
+       ——▶ justfile:1:5
+        │
+      1 │ mod foo
+        │     ^^^
       ",
     )
     .run();
@@ -373,15 +457,51 @@ fn missing_module_file_error() {
     .stderr(
       "
       error: Could not find source file for module `foo`.
-       --> justfile:1:5
-        |
-      1 | mod foo
-        |     ^^^
+       ——▶ justfile:1:5
+        │
+      1 │ mod foo
+        │     ^^^
       ",
     )
     .run();
 }
 
+#[test]
+fn missing_optional_modules_do_not_trigger_error() {
+  Test::new()
+    .justfile(
+      "
+        mod? foo
+
+        bar:
+          @echo BAR
+      ",
+    )
+    .test_round_trip(false)
+    .arg("--unstable")
+    .stdout("BAR\n")
+    .run();
+}
+
+#[test]
+fn missing_optional_modules_do_not_conflict() {
+  Test::new()
+    .justfile(
+      "
+        mod? foo
+        mod? foo
+        mod foo 'baz.just'
+      ",
+    )
+    .write("baz.just", "baz:\n @echo BAZ")
+    .test_round_trip(false)
+    .arg("--unstable")
+    .arg("foo")
+    .arg("baz")
+    .stdout("BAZ\n")
+    .run();
+}
+
 #[test]
 fn list_displays_recipes_in_submodules() {
   Test::new()
@@ -478,6 +598,22 @@ fn modules_with_paths_are_dumped_correctly() {
     .run();
 }
 
+#[test]
+fn optional_modules_with_paths_are_dumped_correctly() {
+  Test::new()
+    .write("commands/foo.just", "foo:\n @echo FOO")
+    .justfile(
+      "
+        mod? foo 'commands/foo.just'
+      ",
+    )
+    .test_round_trip(false)
+    .arg("--unstable")
+    .arg("--dump")
+    .stdout("mod? foo 'commands/foo.just'\n")
+    .run();
+}
+
 #[test]
 fn recipes_may_be_named_mod() {
   Test::new()
@@ -548,3 +684,54 @@ fn module_paths_beginning_with_tilde_are_expanded_to_homdir() {
     .env("HOME", "foobar")
     .run();
 }
+
+#[test]
+fn module_recipe_list_alignment_ignores_private_recipes() {
+  Test::new()
+    .write(
+      "foo.just",
+      "
+# foos
+foo:
+ @echo FOO
+
+[private]
+barbarbar:
+ @echo BAR
+
+@_bazbazbaz:
+ @echo BAZ
+      ",
+    )
+    .justfile("mod foo")
+    .test_round_trip(false)
+    .arg("--unstable")
+    .arg("--list")
+    .stdout(
+      "Available recipes:
+    foo:
+        foo # foos
+",
+    )
+    .run();
+}
+
+#[test]
+fn recipes_with_same_name_are_both_run() {
+  Test::new()
+    .write("foo.just", "bar:\n @echo MODULE")
+    .justfile(
+      "
+        mod foo
+
+        bar:
+          @echo ROOT
+      ",
+    )
+    .test_round_trip(false)
+    .arg("--unstable")
+    .arg("foo::bar")
+    .arg("bar")
+    .stdout("MODULE\nROOT\n")
+    .run();
+}
